// Generated by CoffeeScript 1.9.2
(function() {
  var $, ATTR_WHITELIST, compile, getPropsFromHTMLNode, highlight, htmlWrapperComponent, isValidDocument, mdast, preprocess, rawValueWrapper, sanitize, toChildren, uuid,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  mdast = require('mdast');

  uuid = require('uuid');

  preprocess = require('./preprocess');

  ATTR_WHITELIST = ['href', 'src', 'target'];

  $ = React.createElement;

  toChildren = function(node, defs, parentKey, tableAlign) {
    var child, i;
    if (tableAlign == null) {
      tableAlign = [];
    }
    return (function() {
      var j, len, ref, results;
      ref = node.children;
      results = [];
      for (i = j = 0, len = ref.length; j < len; i = ++j) {
        child = ref[i];
        results.push(compile(child, defs, parentKey + '_' + i, tableAlign));
      }
      return results;
    })();
  };

  isValidDocument = function(doc) {
    var parsererrorNS;
    parsererrorNS = (new DOMParser()).parseFromString('INVALID', 'text/xml').getElementsByTagName("parsererror")[0].namespaceURI;
    return doc.getElementsByTagNameNS(parsererrorNS, 'parsererror').length === 0;
  };

  getPropsFromHTMLNode = function(node, attrWhitelist) {
    var attr, attrs, doc, i, j, parser, props, ref, ref1, string;
    string = node.subtype === 'folded' ? node.startTag.value + node.endTag.value : node.subtype === 'void' ? node.value : null;
    if (string == null) {
      return null;
    }
    parser = new DOMParser();
    doc = parser.parseFromString(string, 'text/html');
    if (!isValidDocument(doc)) {
      return null;
    }
    attrs = doc.body.firstElementChild.attributes;
    props = {};
    for (i = j = 0, ref = attrs.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
      attr = attrs.item(i);
      if ((attrWhitelist == null) || (ref1 = attr.name, indexOf.call(attrWhitelist, ref1) >= 0)) {
        props[attr.name] = attr.value;
      }
    }
    return props;
  };

  sanitize = null;

  highlight = null;

  compile = function(node, defs, parentKey, tableAlign) {
    var className, def, items, j, k, key, l, len, len1, props, ref, ref1, ref2, title;
    if (parentKey == null) {
      parentKey = '_start';
    }
    if (tableAlign == null) {
      tableAlign = null;
    }
    key = parentKey + '_' + node.type;
    switch (node.type) {
      case 'text':
        return rawValueWrapper(node.value);
      case 'escape':
        return '\\';
      case 'break':
        return $('br', {
          key: key
        });
      case 'horizontalRule':
        return $('hr', {
          key: key
        });
      case 'image':
        return $('img', {
          key: key,
          src: node.src,
          title: node.title,
          alt: node.alt
        });
      case 'inlineCode':
        return $('code', {
          key: key,
          className: 'inlineCode'
        }, node.value);
      case 'code':
        return highlight(node.value, node.lang, key);
      case 'root':
        return $('div', {
          key: key
        }, toChildren(node, defs, key));
      case 'strong':
        return $('strong', {
          key: key
        }, toChildren(node, defs, key));
      case 'emphasis':
        return $('em', {
          key: key
        }, toChildren(node, defs, key));
      case 'delete':
        return $('s', {
          key: key
        }, toChildren(node, defs, key));
      case 'paragraph':
        return $('p', {
          key: key
        }, toChildren(node, defs, key));
      case 'link':
        return $('a', {
          key: key,
          href: node.href,
          title: node.title
        }, toChildren(node, defs, key));
      case 'heading':
        return $('h' + node.depth.toString(), {
          key: key
        }, toChildren(node, defs, key));
      case 'list':
        return $((node.ordered ? 'ol' : 'ul'), {
          key: key
        }, toChildren(node, defs, key));
      case 'listItem':
        className = node.checked === true ? 'checked' : node.checked === false ? 'unchecked' : '';
        return $('li', {
          key: key,
          className: className
        }, toChildren(node, defs, key));
      case 'blockquote':
        return $('blockquote', {
          key: key
        }, toChildren(node, defs, key));
      case 'linkReference':
        for (j = 0, len = defs.length; j < len; j++) {
          def = defs[j];
          if (def.type === 'definition' && def.identifier === node.identifier) {
            return $('a', {
              key: key,
              href: def.link,
              title: def.title
            }, toChildren(node, defs, key));
          }
        }
        if (node.referenceType === 'full') {
          return $('span', {
            key: key
          }, ['[', toChildren(node, defs, key), ']', "[" + node.identifier + "]"]);
        } else {
          return $('span', {
            key: key
          }, ['[', toChildren(node, defs, key), ']']);
        }
        break;
      case 'footnoteReference':
        title = '';
        for (l = 0, len1 = defs.length; l < len1; l++) {
          def = defs[l];
          if (def.footnoteNumber === node.footnoteNumber) {
            title = (ref = def.link) != null ? ref : "...";
            return $('sup', {
              key: key,
              id: "fnref" + node.footnoteNumber
            }, [
              $('a', {
                key: key + '-a',
                href: "#fn" + node.footnoteNumber,
                title: title
              }, "" + node.footnoteNumber)
            ]);
          }
        }
        return $('span', {
          key: key
        }, "[^" + node.identifier + "]");
      case 'footnoteDefinitionCollection':
        items = node.children.map(function(def, i) {
          var defBody, k, para;
          k = key + '-ol-li' + i;
          defBody = null;
          if (def.children != null) {
            if ((para = def.children[def.children.length - 1]).type === 'paragraph') {
              para.children.push({
                type: 'text',
                value: ' '
              });
              para.children.push({
                type: 'link',
                href: "#fnref" + def.footnoteNumber,
                children: [
                  {
                    type: 'text',
                    value: '↩'
                  }
                ]
              });
            }
            defBody = toChildren(def, defs, key);
          } else {
            defBody = $('p', {
              key: k + '-p'
            }, [
              def.link, ' ', $('a', {
                key: k + '-p-a',
                href: "#fnref" + def.footnoteNumber
              }, '↩')
            ]);
          }
          return $('li', {
            key: k,
            id: "fn" + def.footnoteNumber
          }, defBody);
        });
        return $('div', {
          key: key,
          "class": 'footnotes'
        }, [
          $('hr', {
            key: key + '-hr'
          }), $('ol', {
            key: key + '-ol'
          }, items)
        ]);
      case 'table':
        return $('table', {
          key: key
        }, toChildren(node, defs, key, node.align));
      case 'tableHeader':
        return $('thead', {
          key: key
        }, [
          $('tr', {
            key: key + '-_inner-tr'
          }, node.children.map(function(cell, i) {
            var k, ref1;
            k = key + '-th' + i;
            return $('th', {
              key: k,
              style: {
                textAlign: (ref1 = tableAlign[i]) != null ? ref1 : 'left'
              }
            }, toChildren(cell, k));
          }))
        ]);
      case 'tableRow':
        return $('tbody', {
          key: key
        }, [
          $('tr', {
            key: key + '-_inner-td'
          }, node.children.map(function(cell, i) {
            var k, ref1;
            k = key + '-td' + i;
            return $('td', {
              key: k,
              style: {
                textAlign: (ref1 = tableAlign[i]) != null ? ref1 : 'left'
              }
            }, toChildren(cell, k));
          }))
        ]);
      case 'tableCell':
        return $('span', {
          key: key
        }, toChildren(node, defs, key));
      case 'html':
        if (node.subtype === 'folded') {
          k = key + '_' + node.tagName;
          props = (ref1 = getPropsFromHTMLNode(node, ATTR_WHITELIST)) != null ? ref1 : {};
          props.key = k;
          return $(node.startTag.tagName, props, toChildren(node, defs, k));
        } else if (node.subtype === 'void') {
          k = key + '_' + node.tagName;
          props = (ref2 = getPropsFromHTMLNode(node, ATTR_WHITELIST)) != null ? ref2 : {};
          props.key = k;
          return $(node.tagName, props);
        } else if (node.subtype === 'special') {
          return $('span', {
            key: key + ':special',
            style: {
              color: 'gray'
            }
          }, node.value);
        } else {
          return $('span', {
            key: key + ':parse-error',
            style: {
              backgroundColor: 'red',
              color: 'white'
            }
          }, node.value);
        }
        break;
      default:
        throw node.type + ' is unsuppoted node type. report to https://github.com/mizchi/md2react/issues';
    }
  };

  htmlWrapperComponent = null;

  rawValueWrapper = null;

  module.exports = function(raw, options) {
    var ast, defs, ref, ref1, ref2, ref3, ref4;
    if (options == null) {
      options = {};
    }
    sanitize = (ref = options.sanitize) != null ? ref : true;
    rawValueWrapper = (ref1 = options.rawValueWrapper) != null ? ref1 : function(text) {
      return text;
    };
    highlight = (ref2 = options.highlight) != null ? ref2 : function(code, lang, key) {
      return $('pre', {
        key: key,
        className: 'code'
      }, [
        $('code', {
          key: key + '-_inner-code'
        }, code)
      ]);
    };
    ast = mdast.parse(raw, options);
    ref3 = preprocess(ast, options), ast = ref3[0], defs = ref3[1];
    ast = (ref4 = typeof options.preprocessAST === "function" ? options.preprocessAST(ast) : void 0) != null ? ref4 : ast;
    return compile(ast, defs);
  };

}).call(this);
