// Generated by CoffeeScript 1.9.2
(function() {
  var ALLOWED_TAG_NAMES, appendFootnoteDefinitionCollection, applyFootnoteNumber, createNodeFromHTMLFragment, decomposeHTMLNode, decomposeHTMLNodes, decomposeHTMLString, defineFootnoteNumber, foldHTMLNodes, isVoidElement, preprocess, removeDefinitions, sanitizeTag, wrapHTMLNodeInParagraph,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  preprocess = function(root, options) {
    var defs, mapping;
    root.children = decomposeHTMLNodes(root.children);
    root.children = foldHTMLNodes(root.children);
    if (options.footnotes) {
      mapping = defineFootnoteNumber(root);
      applyFootnoteNumber(root, mapping);
    }
    defs = removeDefinitions(root);
    if (options.footnotes) {
      appendFootnoteDefinitionCollection(root, defs);
    }
    root = wrapHTMLNodeInParagraph(root);
    root = sanitizeTag(root);
    return [root, defs];
  };

  defineFootnoteNumber = function(node, num, mapping) {
    var child, i, id, len1, ref;
    if (num == null) {
      num = 1;
    }
    if (mapping == null) {
      mapping = {};
    }
    if (node.children == null) {
      return {};
    }
    ref = node.children;
    for (i = 0, len1 = ref.length; i < len1; i++) {
      child = ref[i];
      if (child.type === 'footnoteReference') {
        id = child.identifier;
        if (mapping[id] == null) {
          mapping[id] = num;
          num += 1;
        }
        child.footnoteNumber = mapping[id];
      }
      defineFootnoteNumber(child, num, mapping);
    }
    return mapping;
  };

  applyFootnoteNumber = function(node, mapping) {
    var child, i, id, isFootnoteDefLike, len1, ref, results;
    if (node.children == null) {
      return;
    }
    ref = node.children;
    results = [];
    for (i = 0, len1 = ref.length; i < len1; i++) {
      child = ref[i];
      isFootnoteDefLike = child.type === 'definition' && /^[^]/.test(child.identifier);
      if (child.type === 'footnoteDefinition' || isFootnoteDefLike) {
        id = isFootnoteDefLike ? child.identifier.slice(1) : child.identifier;
        child.footnoteNumber = mapping[id] || 0;
      }
      results.push(applyFootnoteNumber(child, mapping));
    }
    return results;
  };

  appendFootnoteDefinitionCollection = function(node, defs) {
    var def, footnoteDefs;
    footnoteDefs = (function() {
      var i, len1, results;
      results = [];
      for (i = 0, len1 = defs.length; i < len1; i++) {
        def = defs[i];
        if ((def.footnoteNumber != null) && def.footnoteNumber > 0) {
          results.push(def);
        }
      }
      return results;
    })();
    footnoteDefs.sort(function(a, b) {
      return a.footnoteNumber - b.footnoteNumber;
    });
    if (footnoteDefs.length > 0) {
      return node.children.push({
        type: 'footnoteDefinitionCollection',
        children: footnoteDefs
      });
    }
  };

  removeDefinitions = function(node) {
    var child, childDefs, children, defs, i, len1, ref, ref1;
    if (node.children == null) {
      return [];
    }
    children = [];
    defs = [];
    ref = node.children;
    for (i = 0, len1 = ref.length; i < len1; i++) {
      child = ref[i];
      if ((ref1 = child.type) === 'definition' || ref1 === 'footnoteDefinition') {
        defs.push(child);
      } else {
        childDefs = removeDefinitions(child);
        Array.prototype.push.apply(defs, childDefs);
        children.push(child);
      }
    }
    node.children = children;
    return defs;
  };

  foldHTMLNodes = function(nodes) {
    var children, folded, i, index, j, len1, len2, node, pNode, processedNodes, startTag, startTagIndex;
    processedNodes = [];
    for (i = 0, len1 = nodes.length; i < len1; i++) {
      node = nodes[i];
      if (node.subtype === 'end') {
        startTagIndex = null;
        for (index = j = 0, len2 = processedNodes.length; j < len2; index = ++j) {
          pNode = processedNodes[index];
          if (pNode.subtype === 'start' && pNode.tagName === node.tagName) {
            startTagIndex = index;
          }
        }
        if (startTagIndex == null) {
          processedNodes.push(node);
          continue;
        }
        startTag = processedNodes[startTagIndex];
        children = processedNodes.splice(startTagIndex).slice(1);
        folded = {
          type: 'html',
          subtype: 'folded',
          tagName: startTag.tagName,
          startTag: startTag,
          endTag: node,
          children: children
        };
        processedNodes.push(folded);
      } else {
        if (node.children != null) {
          node.children = foldHTMLNodes(node.children);
        }
        processedNodes.push(node);
      }
    }
    return processedNodes;
  };

  decomposeHTMLNodes = function(nodes) {
    var fragmentNodes, i, len1, node, processedNodes;
    processedNodes = [];
    for (i = 0, len1 = nodes.length; i < len1; i++) {
      node = nodes[i];
      if (node.type === 'html') {
        fragmentNodes = decomposeHTMLNode(node);
        if (fragmentNodes != null) {
          Array.prototype.push.apply(processedNodes, fragmentNodes);
        } else {
          node.subtype = 'malformed';
          processedNodes.push(node);
        }
      } else {
        if (node.children != null) {
          node.children = decomposeHTMLNodes(node.children);
        }
        processedNodes.push(node);
      }
    }
    return processedNodes;
  };

  decomposeHTMLNode = function(node) {
    var fragments, value;
    value = node.value;
    if (node.position.start.line === node.position.end.line) {
      value = value.replace(/\n\n/, '');
    }
    fragments = decomposeHTMLString(value);
    return fragments != null ? fragments.map(createNodeFromHTMLFragment) : void 0;
  };

  decomposeHTMLString = function(str) {
    var matches, sumLength;
    if (str === '') {
      return null;
    }
    matches = str.match(/<[^>]*>|[^<>]+/g);
    sumLength = matches.reduce((function(len, s) {
      return len + s.length;
    }), 0);
    if (sumLength !== str.length) {
      return null;
    } else {
      return matches;
    }
  };

  createNodeFromHTMLFragment = function(str) {
    var i, name, ref, ref1, slash, subtype;
    if (/^[^<]/.test(str)) {
      return {
        type: 'text',
        value: str
      };
    }
    ref1 = (ref = /^<(\/?)([0-9A-Z]+)/i.exec(str)) != null ? ref : [], i = ref1.length - 2, slash = ref1[i++], name = ref1[i++];
    subtype = name == null ? 'special' : slash === '/' ? 'end' : isVoidElement(name) ? 'void' : 'start';
    return {
      type: 'html',
      subtype: subtype,
      tagName: name,
      value: str
    };
  };

  isVoidElement = function(elementName) {
    var voidElementNames;
    voidElementNames = ['area', 'base', 'br', 'col', 'command', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr'];
    return voidElementNames.indexOf(elementName) !== -1;
  };

  wrapHTMLNodeInParagraph = function(root) {
    var child, children, i, len1, ref;
    children = [];
    ref = root.children;
    for (i = 0, len1 = ref.length; i < len1; i++) {
      child = ref[i];
      if (child.type === 'html') {
        children.push({
          type: 'paragraph',
          children: [child]
        });
      } else {
        children.push(child);
      }
    }
    root.children = children;
    return root;
  };

  ALLOWED_TAG_NAMES = ['a', 'abbr', 'b', 'br', 'cite', 'code', 'del', 'dfn', 'em', 'i', 'img', 'input', 'ins', 'kbd', 'mark', 'ruby', 'rp', 'rt', 'q', 's', 'samp', 'small', 'span', 'strong', 'sub', 'sup', 'u', 'wbr'];

  sanitizeTag = function(node) {
    var child, children, i, len1, ref, ref1;
    if (node.children == null) {
      return node;
    }
    children = [];
    ref = node.children;
    for (i = 0, len1 = ref.length; i < len1; i++) {
      child = ref[i];
      if (child.subtype === 'folded' && (ref1 = child.tagName, indexOf.call(ALLOWED_TAG_NAMES, ref1) < 0)) {
        children.push(child.startTag);
        Array.prototype.push.apply(children, sanitizeTag(child).children);
        children.push(child.endTag);
      } else {
        children.push(sanitizeTag(child));
      }
    }
    node.children = children;
    return node;
  };

  module.exports = preprocess;

}).call(this);
